package de.tu_darmstadt.tk.SmartHomeNetworkSim.control;

import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.ConfigurationManager;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;

/**
 * Controller which allows access to the configuration
 *
 * @author Andreas T. Meyer-Berg
 */
public class ConfigurationController {

	/**
	 * Model which is configured
	 */
	private Model model;
	/**
	 * Controller
	 */
	Controller controller;
	
	/**
	 * 
	 * @param model model which stores the configuration
	 */
	public ConfigurationController(Model model, Controller controller) {
		this.model = model;
		this.controller = controller;
	}
	
	/**
	 * Returns the ConfigurationManager
	 * 
	 * @return ConfigurationManager of the program
	 */
	public ConfigurationManager getConfigurationManager(){
		return model.getConfigurator();
	}
	
	/**
	 * @return the showConnections
	 */
	public boolean isShowConnections() {
		return model.getConfigurator().getVisualizationConfiguration().isShowConnections();
	}

	/**
	 * @param showConnections the showConnections to set
	 */
	public void setShowConnections(boolean showConnections) {
		model.getConfigurator().getVisualizationConfiguration().setShowConnections(showConnections);
		controller.notifyObservers();
	}

	/**
	 * @return the showLinks
	 */
	public boolean isShowLinks() {
		return model.getConfigurator().getVisualizationConfiguration().isShowLinks();
	}

	/**
	 * @param showLinks the showLinks to set
	 */
	public void setShowLinks(boolean showLinks) {
		model.getConfigurator().getVisualizationConfiguration().setShowLinks(showLinks);
		controller.notifyObservers();
	}

	/**
	 * @return the showSmartDevices
	 */
	public boolean isShowSmartDevices() {
		return model.getConfigurator().getVisualizationConfiguration().isShowSmartDevices();
	}

	/**
	 * @param showSmartDevices the showSmartDevices to set
	 */
	public void setShowSmartDevices(boolean showSmartDevices) {
		model.getConfigurator().getVisualizationConfiguration().setShowSmartDevices(showSmartDevices);
		controller.notifyObservers();
	}

	/**
	 * @return the showSmartDeviceNames
	 */
	public boolean isShowSmartDeviceNames() {
		return model.getConfigurator().getVisualizationConfiguration().isShowSmartDeviceNames();
	}

	/**
	 * @param showSmartDeviceNames the showSmartDeviceNames to set
	 */
	public void setShowSmartDeviceNames(boolean showSmartDeviceNames) {
		model.getConfigurator().getVisualizationConfiguration().setShowSmartDeviceNames(showSmartDeviceNames);
		controller.notifyObservers();
	}

	/**
	 * @return the showTerminatedConnections
	 */
	public boolean isShowTerminatedConnections() {
		return model.getConfigurator().getVisualizationConfiguration().isShowTerminatedConnections();
	}

	/**
	 * @param showTerminatedConnections the showTerminatedConnections to set
	 */
	public void setShowTerminatedConnections(boolean showTerminatedConnections) {
		model.getConfigurator().getVisualizationConfiguration().setShowTerminatedConnections(showTerminatedConnections);
		controller.notifyObservers();
	}

	/**
	 * @return the deviceVisualizationRadius
	 */
	public int getDeviceVisualizationRadius() {
		return model.getConfigurator().getVisualizationConfiguration().getDeviceVisualizationRadius();
	}

	/**
	 * @param deviceVisualizationRadius the deviceVisualizationRadius to set
	 */
	public void setDeviceVisualizationRadius(int deviceVisualizationRadius) {
		model.getConfigurator().getVisualizationConfiguration().setDeviceVisualizationRadius(deviceVisualizationRadius);
		controller.notifyObservers();
	}
	
}
