package de.tu_darmstadt.tk.SmartHomeNetworkSim.view.popups;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Collection;
import java.util.LinkedList;
import java.util.concurrent.ThreadLocalRandom;




import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.Controller;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.control.ImportController;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Connection;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.ConnectionPerformance;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Link;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Model;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Port;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.Protocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.SmartDevice;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.protocols.MQTT_protocol;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.core.simpleImplementation.SimpleLink;
import de.tu_darmstadt.tk.SmartHomeNetworkSim.view.util.JavaFileFilter;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

@SuppressWarnings("serial")
public class ConnectionCreationPanel extends JScrollPane {
	/**
	 * 
	 */
	JPanel content;

	/**
	 * Textfield which stores the Name of the connection
	 */
	private JTextField tfName;

	/**
	 * Parent window which contains this frame - for the close operation
	 */
	private Window frame;

	/**
	 * Connection which is edited
	 */
	private Connection connection;
	
	/**
	 * Index of the disconnected option in the boxes Array 
	 */
	private int disconnectedIndex;

	/**
	 * Ports which could participate in the connection
	 */
	private Port[] ports;

	/**
	 * ComboBoxes to allow role changes of the different Ports
	 */
	private JComboBox<String>[] boxes;

	/**
	 * True if an existing connection is being edited, or false if a new one is
	 * being created
	 */
	private boolean edit;

	/**
	 * Last index which was selected in the linkType ComboBox, to allow restore
	 * the selected Index in case of invalid protocols
	 */
	private int lastIndex = 0;

	/**
	 * Mutex for the protocol change, so the different Devices don't change
	 * their role, while their comboBoxes are updated. True if the protocol
	 * change is in progress and the boxes should not update their role.
	 */
	private boolean protocolChange = false;
	/**
	 * Controller to
	 */
	private Controller controller;

	/**
	 * List of available protocols -> should be moved to the Model and accessed
	 * via controller
	 */
	private LinkedList<Class<? extends Protocol>> availableProtocols;
	/**
	 * List of available connections
	 */
	private LinkedList<Class<? extends Connection>> availableConnection;
	/**
	 * Textfield for the packet loss
	 */
	private JTextField tfPacketLossRate;

	/**
	 * @wbp.parser.constructor
	 */
	public ConnectionCreationPanel(Connection connection, Controller controller) {
		this.controller = controller;
		this.connection = connection;
		ports = new Port[connection.getParticipants().size()];
		int i = 0;
		for (Port d : connection.getParticipants()) {
			// controller.addLinkToDevice(connection, d);
			this.ports[i++] = d;
		}
		edit = true;
		initializePanel();
	}

	public ConnectionCreationPanel(Collection<Port> ports, Link l,
			Controller controller) {
		this.controller = controller;
		connection = new ConnectionPerformance(l, new MQTT_protocol());
		this.ports = new Port[ports.size()];
		int i = 0;
		for (Port d : ports) {
			// controller.addLinkToDevice(connection, d);
			this.ports[i++] = d;
		}
		edit = false;
		initializePanel();
	}

	@SuppressWarnings("unchecked")
	private void initializePanel() {
		disconnectedIndex = connection.getProtocol().getNumberOfRoles();
		
		availableProtocols = controller.getControllerImport().getProtocols();
		availableConnection = controller.getControllerImport().getConnections();

		setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		this.setPreferredSize(new Dimension(600, 220 + ports.length * 20));
		content = new JPanel();
		content.setPreferredSize(new Dimension(600, 200 + ports.length * 20));
		this.setViewportView(content);
		content.setLayout(null);

		JLabel lblDescription = new JLabel("Connection Creation");
		lblDescription.setBounds(12, 0, 144, 16);
		content.add(lblDescription);

		JLabel lblName = new JLabel("Name:");
		lblName.setHorizontalAlignment(SwingConstants.RIGHT);
		lblName.setBounds(10, 10, 140, 20);
		content.add(lblName);

		tfName = new JTextField();
		tfName.setText(connection.getName());
		tfName.setBounds(155, 10, 290, 20);
		content.add(tfName);
		tfName.setColumns(10);
		tfName.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				connection.setName(tfName.getText());
			}
		});
		
		JLabel lblSelectedLink = new JLabel("Selected link:");
		lblSelectedLink.setHorizontalAlignment(SwingConstants.RIGHT);
		lblSelectedLink.setBounds(10, 40, 140, 20);
		content.add(lblSelectedLink);
		
		JComboBox<String> cmbSelectedLink = new JComboBox<String>();
		cmbSelectedLink.setBounds(155, 40, 125, 20);
		content.add(cmbSelectedLink);
		
		JButton btnCreateLink = new JButton("Create Link");
		btnCreateLink.setBounds(290, 40, 155, 20);
		content.add(btnCreateLink);
		
		JLabel lblConnectionType = new JLabel("Connection type:");
		lblConnectionType.setHorizontalAlignment(SwingConstants.RIGHT);
		lblConnectionType.setBounds(10, 70, 140, 20);
		content.add(lblConnectionType);
		
		JComboBox<String> cmbConnection = new JComboBox<String>();
		cmbConnection.setBounds(155, 70, 125, 20);
		content.add(cmbConnection);
		
		for (int i = 0; i < availableConnection.size(); i++)
			cmbConnection.addItem(availableConnection.get(i).getSimpleName());

		cmbConnection.setBounds(155, 70, 125, 20);
		content.add(cmbConnection);
		// Set Index to selected Protocol
		for (int i = 0; i < availableConnection.size(); i++)
			if (connection.getClass()
					.equals(availableConnection.get(i))){
				//Select the right protocol and save last index
				cmbConnection.setSelectedIndex(i);
			}
		
		JButton btnImportConnection = new JButton("Import Connection");
		btnImportConnection.setBounds(290, 70, 155, 20);
		content.add(btnImportConnection);

		JLabel lblProtocolType = new JLabel("Protocol Type:");
		lblProtocolType.setHorizontalAlignment(SwingConstants.RIGHT);
		lblProtocolType.setBounds(10, 100, 140, 20);
		content.add(lblProtocolType);

		JComboBox<String> cmbProtocolType = new JComboBox<String>();
		for (int i = 0; i < availableProtocols.size(); i++)
			try {
				cmbProtocolType.addItem(availableProtocols.get(i).newInstance()
						.getName());
			} catch (InstantiationException | IllegalAccessException e1) {
				System.out.println("Protocol " + i + " is invalid");
				cmbProtocolType.addItem("unknown");
			}
		cmbProtocolType.setBounds(155, 100, 125, 20);
		content.add(cmbProtocolType);
		// Set Index to selected Protocol
		for (int i = 0; i < availableProtocols.size(); i++)
			if (connection.getProtocol().getClass()
					.equals(availableProtocols.get(i))){
				//Select the right protocol and save last index
				cmbProtocolType.setSelectedIndex(i);
				lastIndex = i;
			}

		// Add Functionality for changing protocol
		cmbProtocolType.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				/**
				 * Old Protocol
				 */
				Protocol oldProtocol = connection.getProtocol();
				//Set Mutex to true, to disallow changes of roles during the editing
				protocolChange = true;
				/**
				 * New Protocol which should be created and configured
				 */
				Protocol newProtocol = null;
				try {
					//Create new Instance of the protocol
					newProtocol = availableProtocols.get(
							cmbProtocolType.getSelectedIndex()).newInstance();
				} catch (InstantiationException | IllegalAccessException e1) {
					System.out
							.println("WARNING: Protocol could not be initialized");
				}
				if (newProtocol == null) {
					cmbProtocolType.setSelectedIndex(lastIndex);
					System.out
							.println("WARNING: Invalid Protocol Selected - restore last index");
				} else if(connection.setProtocol(newProtocol)){
					/**
					 * New Roles as Strings
					 */
					String[] newRoles = newProtocol.getRoles();
					/**
					 * The old disconnected status
					 */
					int oldDisconnected = oldProtocol.getNumberOfRoles();
					//Update to new disconnected Index
					disconnectedIndex = newProtocol.getNumberOfRoles();
					//Update Protocol
						
					// Update lastIndex to new Index
					lastIndex = cmbProtocolType.getSelectedIndex();
					//Update Boxes
					for (int i = 0; i < boxes.length; i++) {
						/**
						 * The previous selected Index
						 */
						int oldSelected = boxes[i].getSelectedIndex();
						//Update Labels of the current Box
						boxes[i].removeAllItems();
						for (int j = 0; j < newRoles.length; j++)
							boxes[i].addItem(newRoles[j]);
						boxes[i].addItem("Disconnected");
						//Try to update the Roles
						if (oldSelected < 0 || oldSelected >= disconnectedIndex
								|| oldSelected == oldDisconnected) {
							//Invalid Index -> Disconnected
							controller.removeDeviceFromConnection(ports[i], connection);
							boxes[i].setSelectedIndex(disconnectedIndex);
						} else {
							if (controller.addDeviceToConnection(ports[i], connection, oldSelected)){
								//Set to Box to display role
								boxes[i].setSelectedIndex(oldSelected);
							} else {
								//Could not be added -> Disconnected
								controller.removeDeviceFromConnection(ports[i], connection);
								boxes[i].setSelectedIndex(disconnectedIndex);
							}
						}
					}
				} else {
					cmbProtocolType.setSelectedIndex(lastIndex);
				}
				//Set Mutex back to false - allow changes
				protocolChange = false;
			}
		});

		JButton btnImportProtocol = new JButton("Import Protocol");
		btnImportProtocol.setBounds(290, 100, 155, 20);
		content.add(btnImportProtocol);
		btnImportProtocol.addActionListener(a -> {
			//Filechooser starting in the base directory
			JFileChooser fc = new JFileChooser(new File(System.getProperty("user.dir")));
			fc.setFileFilter(new JavaFileFilter());
			int returnVal = fc.showOpenDialog(this);

	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	        	/**
	        	 * selected File
	        	 */
	            File file = fc.getSelectedFile();
	            String[] parts = file.getName().split("[.]");
	            if(parts.length <= 1)
	            	return;
	            /**
	             * Check the file type
	             */
	            String ending = parts[parts.length-1].toLowerCase();
	            if(ending.equals("java")){
	            	
	            	// Compile source file.
	            	/**
	            	 * Imported Java Class
	            	 */
	            	Class<? extends Protocol> imported = null;
	            	try {
						imported = (Class<? extends Protocol>) ImportController.importJavaClass(file);
					} catch (Exception e) {
						e.printStackTrace();
					}
	            	if(imported==null){
	            		System.out.println("WARNING: Importing failed");
	            	}else{
	            		@SuppressWarnings("rawtypes")
						Class[] interfaces = imported.getInterfaces();
	            		boolean isProtocol = false;
	            		for(int i = 0; i<interfaces.length; i++){
	            			if(interfaces[i] == Protocol.class)
	            				isProtocol = true;
	            		}
	            		if(!isProtocol){
	            			System.out.println("WARNING: No valid protocol");
	            		}
	            		//Add Protocol
	            		if(controller.getControllerImport().addProtocol(imported)){
	            			//Update GUI
	            			try {
								cmbProtocolType.addItem((imported.newInstance()).getName());
								availableProtocols.add(imported);
							} catch (Exception e1) {
								System.out.println("Adding Protocol to the Links failed");
							}
	            		}else{
	            			System.out.println("Importing into the model failed");
	            		}
	            	}
	            }else{
	            	System.out.println("WARNING: Invalid File Type");
	            	return;
	            }
	        } else {
	           //Importing Cancelled by user
	        }
		});
		
		JLabel lblPacketlossrate = new JLabel("PacketLossRate:");
		lblPacketlossrate.setHorizontalAlignment(SwingConstants.RIGHT);
		lblPacketlossrate.setBounds(10, 130, 140, 20);
		lblPacketlossrate.setToolTipText("Probability of Packet loss, should be a Double in [0.0,1.0]");
		content.add(lblPacketlossrate);
		
		tfPacketLossRate = new JTextField();
		tfPacketLossRate.setBounds(155, 130, 90, 20);
		content.add(tfPacketLossRate);
		tfPacketLossRate.setColumns(10);
		tfPacketLossRate.setText(""+connection.getPacketLossProbability());
		tfPacketLossRate.getDocument().addDocumentListener(new DocumentListener() {
			
			@Override
			public void removeUpdate(DocumentEvent e) {
				updateConnectionAndField();
			}
			
			@Override
			public void insertUpdate(DocumentEvent e) {
				updateConnectionAndField();
			}
			
			@Override
			public void changedUpdate(DocumentEvent e) {
				updateConnectionAndField();
			}
			
			private void updateConnectionAndField(){
				try{
					double newRate = Double.parseDouble(tfPacketLossRate.getText());
					if(newRate >= 0.0 && newRate <= 1.0){
						connection.setPacketLossProbability(newRate);
						tfPacketLossRate.setBackground(Color.WHITE);
					}else{
						tfPacketLossRate.setBackground(Color.RED);
					}
				}catch(Exception exception){
					tfPacketLossRate.setBackground(Color.RED);					
				}
			}
		});
		
		JLabel lblStatus = new JLabel("Status:");
		lblStatus.setHorizontalAlignment(SwingConstants.RIGHT);
		lblStatus.setBounds(250, 130, 60, 20);
		content.add(lblStatus);
		
		JComboBox<String> cmbStatus = new JComboBox<String>();
		for(int i = 0; i<5; i++)
			cmbStatus.addItem(Connection.getStatusName((byte)i));
		cmbStatus.setBounds(320, 130, 125, 20);
		cmbStatus.setSelectedIndex(connection.getStatus());
		content.add(cmbStatus);
		cmbStatus.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if(cmbStatus.getSelectedIndex()<0||cmbStatus.getSelectedIndex()>4)
					cmbStatus.setSelectedIndex(connection.getStatus());
				else
					connection.setStatus((byte) cmbStatus.getSelectedIndex());
				
			}
		});
		
				JButton btnCreate = new JButton("Verify and Create");
				btnCreate.setBounds(125, 160, 206, 25);
				content.add(btnCreate);
				
				btnCreate.addActionListener(new ActionListener() {

					@Override
					public void actionPerformed(ActionEvent e) {
						
						if(!edit){ 
							if(!connection.getLink().getConnections().contains(connection))
							connection.getLink().addConnection(connection);
							for(int i = 0; i<ports.length; i++){
								if(!ports[i].getOwner().getLinks().contains(connection.getLink()))
									ports[i].getOwner().addLink(connection.getLink());
								if(connection.getProtocol().getDevices().contains(ports[i])){
									if(!connection.getParticipants().contains(ports[i]))
										connection.addSmartDevice(ports[i]);
									if(ports[i].getConnection()!=connection)
										ports[i].setConnection(connection);
									if(!ports[i].getOwner().getPorts().contains(ports[i]))
										ports[i].getOwner().addPort(ports[i]);
								} else {
									if(connection.getParticipants().contains(ports[i]))
										connection.removeSmartDevice(ports[i]);
									if(ports[i].getConnection()!=null)
										ports[i].setConnection(null);
									if(ports[i].getOwner().getPorts().contains(ports[i]))
										ports[i].getOwner().removePort(ports[i]);
									
								}
							}
							if(!controller.getLinks().contains(connection.getLink()))
								controller.addLink(connection.getLink());
							if(!controller.getConnections().contains(connection))
								controller.addConnection(connection);
						}
						 
						connection.setName(tfName.getText());
						content.setVisible(false);
						setVisible(false);
						if (frame != null) {
							frame.setVisible(false);
							frame.dispose();
						}
						controller.notifyObservers();
					}
				});
		
		/**
		 * Height of the current Box which is being created
		 */
		int currentHeight = 190;

		boxes = new JComboBox[ports.length];
		for (int i = 0; i < ports.length; i++) {
			//Effectively final variables for the action listener
			int pos = i;
			Port currentPort = ports[i];
			SmartDevice currentDevice = currentPort.getOwner();
			
			JLabel lblDevice = new JLabel(currentDevice.getName() + ":"
					+ currentPort.getPortNumber() + ":");
			lblDevice.setHorizontalAlignment(SwingConstants.RIGHT);
			lblDevice.setBounds(10, currentHeight, 230, 18);
			content.add(lblDevice);

			boxes[pos] = new JComboBox<String>();
			for (String role : connection.getProtocol().getRoles())
				boxes[pos].addItem(role);
			boxes[pos].addItem("Disconnected");
			boxes[pos].setBounds(250, currentHeight, 240, 18);
			content.add(boxes[pos]);
			if(edit){
				int roleOfDevice = connection.getProtocol().getRoleOfDevice(currentPort);
				if(roleOfDevice != -1)
					boxes[pos].setSelectedIndex(roleOfDevice);
				else
					boxes[pos].setSelectedIndex(disconnectedIndex);
			} else {
				/**
				 * Number of tries to add the Port
				 */
				int n = 5;
				for(int j = 0; j <= n; j++){
					if(j == 0){//Try to add Router first
						if(controller.addDeviceToConnection(currentPort, connection, 0)){
							boxes[pos].setSelectedIndex(0);
							break;
						}										
					}else if(j == n){//If it could not be added
						boxes[pos].setSelectedIndex(disconnectedIndex);
						controller.removeDeviceFromConnection(currentPort, connection);
						break;
					}else{
						int randomRole = ThreadLocalRandom.current().nextInt(0, disconnectedIndex);
						if(controller.addDeviceToConnection(currentPort, connection, randomRole)){
							boxes[pos].setSelectedIndex(randomRole);
							break;
						}
					}
				}
			}
			boxes[pos].addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					if (protocolChange) {
						return;
					}
					/**
					 * Selected Index of the ComboBox, which represents the new role (or disconnected)
					 */
					int selected = boxes[pos].getSelectedIndex();
					/**
					 * True if role was successfully changed
					 */
					boolean successfullChange = controller.changeRoleOfDevice(connection.getProtocol(),connection,
							ports[pos], selected);
					/**
					 * Set to Disconnected, if role could not be changed
					 */
					if(!successfullChange){
						boxes[pos].setSelectedIndex(connection.getProtocol().getNumberOfRoles());
					}	
				}
			});
			currentHeight += 20;
		}
	}

	/**
	 * Test the Panel
	 */
	private static void testGUI() {
		JFrame frame = new JFrame("Connection Creation Panel");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		ConnectionCreationPanel panel;
		Link testNet = new SimpleLink("Test Network");
		LinkedList<SmartDevice> devicesOfLink = new LinkedList<SmartDevice>();
		LinkedList<Port> ports = new LinkedList<Port>();
		for (int i = 0; i < 5; i++) {
			SmartDevice d = new SmartDevice("Device" + i);
			Port p = new Port(d, (short) i);
			d.addPort(p);
			devicesOfLink.add(d);
			ports.add(p);
		}
		panel = new ConnectionCreationPanel(ports, testNet, new Controller(
				new Model()));
		panel.setFrame(frame);
		frame.setContentPane(panel);
		frame.pack();
		frame.setVisible(true);
	}

	public static void main(String[] args) {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				testGUI();
			}
		});
	}

	/**
	 * @return the frame
	 */
	public Window getFrame() {
		return frame;
	}

	/**
	 * @param frame
	 *            the frame to set
	 */
	public void setFrame(Window frame) {
		this.frame = frame;
	}
}